/*
Simple HTTP client from Harry Fairhead 
Master the Raspberry Pi Pico in C: WiFi with lwIP & mbedtls
IO Press, 2023, page 42
*/

#include <stdio.h>
#include "pico/stdlib.h"
#include "pico/cyw43_arch.h"
#include "setupWifi.h"
#include "lwip/tcp.h"
#include <pico/multicore.h>
#include "hardware/sync.h"
#include "hardware/adc.h"

// ======================
// protothreads header
#include "pt_cornell_rp2040_v1_3.h"

bool data_ready ;
bool cmd_ready, page_done, enable_reload ;
int led_state, reload_time = 1000000 ;
float temp, client_ch0_voltage ;

//=======================
// network stuff
#define BUF_SIZE 2048
char myBuff[BUF_SIZE];
//char header[] = "GET /ledtest?led=0&test=100&test2=90 HTTP/1.1\r\n\r\n "; //HOST:192.168.4.1\r\nConnection: close\r\n\r\n";&test=100&test2=90
char header[128] = "GET /ledtest HTTP/1.1\r\n\r\n ";
char site[32] ;

err_t recv(void *arg, struct tcp_pcb *pcb, struct pbuf *p, err_t err)
{
 if (p != NULL)
 {
    printf("recv total %d  this buffer %d next %d err %d\n", p->tot_len, p->len, p->next, err);
    pbuf_copy_partial(p, myBuff, p->tot_len, 0);
    myBuff[p->tot_len] = 0;
    //printf("Buffer= %s\n", myBuff);
    tcp_recved(pcb, p->tot_len);
    pbuf_free(p);
    data_ready = true ;
 }
    return ERR_OK;
}

static err_t connected(void *arg, struct tcp_pcb *pcb, err_t err)
{
  err = tcp_write(pcb, header, strlen(header), 0);
  //if(err == ERR_OK) printf("err OK write\r\n");
  err = tcp_output(pcb);
  return 0;
}

// ====================================
// tcp pcb states
// enum tcp_state {
//   CLOSED      = 0,
//   LISTEN      = 1,
//   SYN_SENT    = 2,
//   SYN_RCVD    = 3,
//   ESTABLISHED = 4,
//   FIN_WAIT_1  = 5,
//   FIN_WAIT_2  = 6,
//   CLOSE_WAIT  = 7,
//   CLOSING     = 8,
//   LAST_ACK    = 9,
//   TIME_WAIT   = 10
// };

// ============================================
// NOTE -- NOTE -- NOTE -- NOTE -- NOTE -- NOTE
// This thread must be highest priority
// Since timimg is critical, no other thread
// on this core should run for long before
// yielding!!!
// ============================================
// handle the server request and print the page
 //struct tcp_pcb *pcb;
static PT_THREAD (protothread_handle_data(struct pt *pt)){
    PT_BEGIN(pt);
    
   // initial display of page 
    static ip_addr_t ip;
    IP4_ADDR(&ip, 192, 168, 4, 1);

    static struct tcp_pcb *pcb  ;
    pcb = tcp_new();
    tcp_recv(pcb, recv);
    
    cyw43_arch_lwip_begin();
    err_t err = tcp_connect(pcb, &ip, 80, connected);
    cyw43_arch_lwip_end();

    //while(!data_ready){};
    PT_YIELD_UNTIL(pt, data_ready);
    data_ready = false ;
    printf("%s\n", myBuff);
    page_done = true ;
    
    // 
    while (true)
    {        
         // build URL
       PT_YIELD_UNTIL(pt, cmd_ready);
       // clear the flag
       cmd_ready = false;

        // init the connection      
        cyw43_arch_lwip_begin();
        //err_t cerr = tcp_close(pcb) ;
         tcp_abort(pcb) ;
        cyw43_arch_lwip_end();
       // pcb = NULL ;
       
        static struct tcp_pcb *pcb  ;
        pcb = tcp_new();
        tcp_recv(pcb, recv);
              
         cyw43_arch_lwip_begin();
        err_t err = tcp_connect(pcb, &ip, 80, connected);        
        cyw43_arch_lwip_end();     

        // wait for recv ISR to finish
        PT_YIELD_UNTIL(pt, data_ready);
        data_ready = false ;
        printf("%s\n", myBuff);
        // signal serial thread
        page_done = true ;

    } // while(1)
    PT_END(pt);
}

// ========================================
// === core 1 serial
// ========================================

static PT_THREAD (protothread_serial(struct pt *pt)){
    PT_BEGIN(pt);
        static char cmd[16], arg1[16], arg2[16] ;
        static char* token ;
        static int led, test, test2 ;
        static int begin_time, response_time ;
        // initial page to get
        // other page is /newpage
        sprintf(site, "/ledtest");

    // print prompt
    while(true){
        // wait for done signal for tcp thread
        PT_YIELD_UNTIL(pt, page_done);
        page_done = false ;

        response_time = (int) PT_GET_TIME_usec() - begin_time ;
        // response time of for debugging
        printf(" server response time %d mSec", response_time/1000);

        // press any key to exit auto-reload
        if(enable_reload && !(int)uart_is_readable(UART_ID)){
            PT_YIELD_usec(reload_time);
           if(strcmp(site,"/ledtest")==0) sprintf(header,"GET %s HTTP/1.1\r\n\r\n ", site) ;
           if(strcmp(site,"/newpage")==0)
           sprintf(header,"GET %s?client_adc=%f&client_adc0=%f HTTP/1.1\r\n\r\n ", site, temp, client_ch0_voltage) ;
        }
        // no autoload, manual command
        else{
            // if we get here, then user pressed a key
            // so disable auto reload
            enable_reload = false ;

            // now get a user command
            printf("\n\r\n\rserver cmd: ") ;
        
            // spawn a thread to do the non-blocking serial read
            serial_read ;
            //parse input
            token = strtok(pt_serial_in_buffer, "  ");
            strcpy(cmd, token) ;
            token = strtok(NULL, "  ");
            strcpy(arg1, token) ;
            token = strtok(NULL, "  ");
            strcpy(arg2, token) ;

            // set up GET command to server
            // enable auto-reload
            if(strcmp(cmd,"reload")==0){
                enable_reload = true ;
                // mSec
                sscanf(arg1, "%d", &reload_time);
                reload_time = reload_time * 1000 ;
            }

            //  is it a page change
            if(strcmp(cmd,"go")==0){
                if(strcmp(arg1,"adcpage")==0){
                    sprintf(site, "/newpage");
                    sprintf(header,"GET %s?client_adc=%f&client_adc0=%f HTTP/1.1\r\n\r\n ", site, temp, client_ch0_voltage) ;                   
                }
                if(strcmp(arg1,"ledpage")==0){
                    sprintf(site, "/ledtest");
                    sprintf(header,"GET /ledtest HTTP/1.1\r\n\r\n ") ;                   
                }
            }
            // is the page /ledtest
            else if(strcmp(site,"/ledtest")==0){
            
                if(strcmp(cmd,"led")==0){
                    if (strcmp(arg1,"on")==0) led=1; else led=0;
                }
                //
                if(strcmp(cmd,"test")==0){
                    sscanf(arg1,"%d", &test);
                }
                //
                if(strcmp(cmd,"test2")==0){
                    sscanf(arg1,"%d", &test2);
                }
                sprintf(header,"GET %s?led=%01d&test=%d&test2=%d HTTP/1.1\r\n\r\n ", site, led, test, test2) ;
            }

            // is the page /adctest
            // 'temp' commnad SENDS temp to server
            else if(strcmp(site,"/newpage")==0){
                if(strcmp(cmd,"temp")==0){
                    //sprintf(header,"GET %s?client_adc=%f HTTP/1.1\r\n\r\n ", site, temp) ; ///?client_adc=%f , temp
                    sprintf(header,"GET %s?client_adc=%f&client_adc0=%f HTTP/1.1\r\n\r\n ", site, temp, client_ch0_voltage) ;
                }
                if(strcmp(cmd,"volts")==0){
                    //sprintf(header,"GET %s?client_adc0=%f HTTP/1.1\r\n\r\n ", site, client_ch0_voltage) ; ///?client_adc=%f , temp
                    sprintf(header,"GET %s?client_adc=%f&client_adc0=%f HTTP/1.1\r\n\r\n ", site, temp, client_ch0_voltage) ;
                }
            }
        }
        // signal tcp thread
        begin_time = (int) PT_GET_TIME_usec();
        cmd_ready = true ;
    }
    PT_END(pt);
}

// =========================================
// adc thread
// =========================================

static PT_THREAD (protothread_temp(struct pt *pt))
{
    PT_BEGIN(pt);
        const float conversion_factor = 3.3f / (1<<12);
        float temp_result ;
        //
        adc_init();
        adc_set_temp_sensor_enabled(true);

        // Make sure GPIO is high-impedance, no pullups etc
        adc_gpio_init(26);

        //
        while(1){
            // conver to volts
            adc_select_input(4);
            temp_result = (float)adc_read() * conversion_factor;
            //printf("adc %d v %f \n", adc_read(), result);
            /// convert volts to ~temp (from hardare manual)
            temp = 27.0f - (temp_result -0.706f)/0.001721f ;
            //
            // channel 0
           adc_select_input(0);
           client_ch0_voltage = (float)adc_read() * conversion_factor;

           // }
           //printf( "Internal Temp = %3.1f C", temp) ;
            PT_YIELD_usec(1000000);

        }
    PT_END(pt);
} // serial thread
// ========================================
// === core 1 main -- started in main below
// ========================================
void core1_main(){ 
  
  //  === add threads  ====================
  // for core 1
  // NOTE first added is HIGHEST priority
  pt_add_thread(protothread_serial) ;
  pt_add_thread(protothread_temp) ;
  //
  // === initalize the scheduler ==========
  pt_sched_method = SCHED_PRIORITY ;
  pt_schedule_start ;
  // NEVER exits
  // ======================================
}

// ========================================
// === core 0 main
// ========================================
int main(){
  // set the clock
  //set_sys_clock_khz(250000, true); // 171us
  // start the serial i/o
  stdio_init_all() ;
  // announce the threader version on system reset
  printf("\n\rProtothreads RP2040 v1.3 two-core, priority\n\r");

  // =======================
  // network connect
  connect();


  // start core 1 threads
   multicore_reset_core1();
   multicore_launch_core1(&core1_main);

  // === config threads ========================
  // for core 0
  // NOTE first added is HIGHEST priority
  pt_add_thread(protothread_handle_data) ;
  //pt_add_thread(protothread_serial) ;
  //
  // === initalize the scheduler ===============
  // method is either:
  //   SCHED_PRIORITY or SCHED_ROUND_ROBIN
  pt_sched_method = SCHED_PRIORITY ;
  pt_schedule_start ;
  // NEVER exits
  // ===========================================
} // end main