/*
Simple HTTP client from Harry Fairhead 
Master the Raspberry Pi Pico in C: WiFi with lwIP & mbedtls
IO Press, 2023, page 42
*/

#include <stdio.h>
#include "pico/stdlib.h"
#include "pico/cyw43_arch.h"
#include "setupWifi.h"
#include "lwip/tcp.h"
#include <pico/multicore.h>
#include "hardware/sync.h"

// ======================
// protothreads header
#include "pt_cornell_rp2040_v1_3.h"

// protothreads signals
struct pt_sem connection_ready_s ;
struct pt_sem receive_ready_s ;

//=======================
// network stuff
#define BUF_SIZE 2048
char myBuff[BUF_SIZE];
char header[] = "GET /ledtest?led=1&test=100&test2=90 HTTP/1.1\r\n\r\n "; //HOST:192.168.4.1\r\nConnection: close\r\n\r\n";
//char header[] = "GET /ledtest HTTP/1.1\r\n\r\n ";

err_t recv(void *arg, struct tcp_pcb *pcb, struct pbuf *p, err_t err)
{
 if (p != NULL)
 {
    printf("recv total %d  this buffer %d next %d err %d\n", p->tot_len, p->len, p->next, err);
    pbuf_copy_partial(p, myBuff, p->tot_len, 0);
    myBuff[p->tot_len] = 0;
    //printf("Buffer= %s\n", myBuff);
    tcp_recved(pcb, p->tot_len);
    pbuf_free(p);
    PT_SEM_SIGNAL(pt, &receive_ready_s) ;
 }
 //else printf("%d\n\r", err);
 
 //   int32_t rssi ;
 //   cyw43_wifi_get_rssi (&cyw43_state, &rssi);
 //   printf("rssi=%d\n\r", rssi) ;
    return ERR_OK;
}

static err_t connected(void *arg, struct tcp_pcb *pcb, err_t err)
{
  PT_SEM_SIGNAL(pt, &connection_ready_s) ;
  printf("connected");
  return 0;
}

// ====================================
// tcp states
// enum tcp_state {
//   CLOSED      = 0,
//   LISTEN      = 1,
//   SYN_SENT    = 2,
//   SYN_RCVD    = 3,
//   ESTABLISHED = 4,
//   FIN_WAIT_1  = 5,
//   FIN_WAIT_2  = 6,
//   CLOSE_WAIT  = 7,
//   CLOSING     = 8,
//   LAST_ACK    = 9,
//   TIME_WAIT   = 10
// };

// ====================================
// handle the server request and print the page
 //struct tcp_pcb *pcb;
static PT_THREAD (protothread_handle_data(struct pt *pt)){
    PT_BEGIN(pt);
 
    //struct tcp_pcb *pcb = tcp_new();
    ip_addr_t ip;
    IP4_ADDR(&ip, 192, 168, 4, 1);

    static struct tcp_pcb *pcb  ;
    pcb = tcp_new();
    tcp_recv(pcb, recv);
    
    cyw43_arch_lwip_begin();
    err_t err = tcp_connect(pcb, &ip, 80, connected);
    cyw43_arch_lwip_end();
    //if(err == ERR_OK) printf("err OK connect\r\n");
    
    printf("connect wait\n\r");
    // wait for the asynch connection
    PT_SEM_WAIT(pt, &connection_ready_s) ;
    printf("connect success\n\r");

    // request page from server on new connection
    cyw43_arch_lwip_begin();
    err = tcp_write(pcb, header, strlen(header), 0);
    err = tcp_output(pcb);
    printf("%d\n\r", err) ;
    cyw43_arch_lwip_end();

    // wait for data
        printf("receive wait\n\r");
        //sleep_ms(2000) ;
    PT_SEM_WAIT(pt, &receive_ready_s) ;
        printf("receive success\n\r");
    
    // print the response page
        printf("%s\n", myBuff);

    //
    while (true)
    { 
       // print prompt
        //sprintf(pt_serial_out_buffer, "server cmd: ") ;
        // spawn a thread to do the non-blocking write
        //serial_write ;

        // spawn a thread to do the non-blocking serial read
       // serial_read ;

         // parse input

         // build URL

        // init the connection
        
        cyw43_arch_lwip_begin();
        //err_t cerr = tcp_close(pcb) ;
         tcp_abort(pcb) ;
        cyw43_arch_lwip_end();
       // pcb = NULL ;

        printf("%d\n\r", pcb->state);        
        sleep_ms(2000) ;
        printf("%d\n\r", pcb->state);
       
        static struct tcp_pcb *pcb  ;
        pcb = tcp_new();
        tcp_recv(pcb, recv);
        printf("%d\n\r", pcb->state);
       
         cyw43_arch_lwip_begin();
        err_t err = tcp_connect(pcb, &ip, 80, connected);
        printf("%d\n\r", pcb->state);
        
        cyw43_arch_lwip_end();
        //if(err == ERR_OK) printf("err OK connect\r\n");
        
        printf("connect wait\n\r");
        // wait for the asynch connection
        PT_SEM_SAFE_WAIT(pt, &connection_ready_s) ;
        printf("connect success\n\r");

        // request page from server on new connection
        cyw43_arch_lwip_begin();
        err = tcp_write(pcb, header, strlen(header), 0);
        err = tcp_output(pcb);
        printf("%d\n\r", err) ;
        cyw43_arch_lwip_end();

        // wait for data
         printf("receive wait\n\r");
         //sleep_ms(2000) ;
        PT_SEM_SAFE_WAIT(pt, &receive_ready_s) ;
         printf("receive success\n\r");
        
        // print the response page
         printf("%s\n", myBuff);

        //  cyw43_arch_lwip_begin();
        //  err_t close_err = tcp_close(pcb);
        //  printf("%d\n\r", close_err);
        //  cyw43_arch_lwip_end();
        
         
         
    } // while(1)
    PT_END(pt);
}

// ========================================
// === core 0 main
// ========================================
int main(){
  // set the clock
  //set_sys_clock_khz(250000, true); // 171us
  // start the serial i/o
  stdio_init_all() ;
  // announce the threader version on system reset
  printf("\n\rProtothreads RP2040 v1.3 two-core, priority\n\r");

  // =======================
  // network connect
  connect();

   // init two semaphores to synch tcp with program
   PT_SEM_INIT(&connection_ready_s, 0) ;
   PT_SEM_INIT(&receive_ready_s, 0) ;

  // start core 1 threads
  // multicore_reset_core1();
  //multicore_launch_core1(&core1_main);

  // === config threads ========================
  // for core 0
  // NOTE first added is HIGHEST priority
  pt_add_thread(protothread_handle_data);
 
  //
  // === initalize the scheduler ===============
  // method is either:
  //   SCHED_PRIORITY or SCHED_ROUND_ROBIN
  pt_sched_method = SCHED_PRIORITY ;
  pt_schedule_start ;
  // NEVER exits
  // ===========================================
} // end main